%--------------------------------------------------------------------------------------------------%
%
% function [f,J] = costfunctionDensOnly(x,exp_t,ic,exp_stress,exp_temp,params,exp_strain)
%   calculates the residual (straub error) and jacobian for the
%   optimization of the density coefficients only
%
%   Parameters:
%       x - design variables (alpha, beta)
%       exp_t - experimental time vectors (cell array)
%       ic - initial conditions (cell array)
%       exp_stress - experimental stress vectors (cell array)
%       params - hem sma parameters (struct)
%       exp_stress - experimental strain vectors (cell array)
%
%   Returns: 
%       f - vector of residuals
%       J - Jacobian
%
%   Required files: subinterval_pts.m, AlphaLogNormal.m, AlphaNormal.m,
%       pxx_he.m, sma_strain.m
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%

function [f,J] = costfunctionDensOnly(x,exp_t,ic,exp_stress,exp_temp,params,exp_strain)

% extract weights
weights = x;

% extract density parameters
sigmaRmean = params.sigmaRmean;
c1 = params.c1;
b1 = params.b1;

nAlpha = length(sigmaRmean);
nBeta = length(b1);

% extract paramters
alpha = weights(1:nAlpha);
beta = weights(nAlpha+1:nAlpha+nBeta);

% range of integration on sigmaR
medians_ = exp(sigmaRmean);
sds_ = sqrt((exp(c1.^2)-1).*exp(2*sigmaRmean+c1.^2));
lbR_ = max(medians_-2.*sds_,zeros(length(sigmaRmean),1));
ubR_ = medians_+2.*sds_;
lbR = min(lbR_);
ubR = max(ubR_);


% range on sigmaI
sigmaImax = 2*max(b1);

% set up intervals
[xR wR] = subinterval_pts(params.Npts,params.NsubInts,lbR,ubR);
[xI wI] = subinterval_pts(params.Npts,params.NsubInts,-sigmaImax,sigmaImax);

% scale points
wR = wR.*(ubR-lbR)/params.NsubInts;
wI = wI*sigmaImax*2/params.NsubInts;
% calculate distributions
v1 = AlphaLogNormal(xR,alpha,sigmaRmean,c1)/(sum(alpha));
v2 = AlphaNormal(xI,beta,b1)/(sum(beta));



% kernels
for i =1:nAlpha
    phi(:,i) = AlphaLogNormal(xR,1,sigmaRmean(i),c1(i));
end
for j=1:nBeta
    psi(:,j) = AlphaNormal(xI,1,b1(j));
end
% initialize jacobian and residuals
ntotal = length(exp_stress{1})+length(exp_stress{2});
f = zeros(ntotal,1);
J = zeros(ntotal,length(x));

offset = 0;
for mm = 1:length(exp_stress)
    
    stress = exp_stress{mm};
    temp = exp_temp{mm};
    x0 = ic{mm};
    t = exp_t{mm};
    % check to make sure stress is a column vector
    [ns ms] = size(stress);
    if(ns == 1)
        stress = stress'; % make a column vector
    end

    % make a matrix
    stress_ = zeros(length(stress),length(xI));

    for i=1:length(xI)
        stress_(:,i) = stress+xI(i);
    end
    
    % size of 3d matrix
    ns = length(t);
    ms = length(xI);
    ks = length(xR);

    % initialize output  (xp, xm, xa)
    xp = zeros(ns,1);
    xa = zeros(ns,1);
    xm = zeros(ns,1);
    strain = zeros(ns,1);
    xp_(1:ms,1:ks) = x0(1);
    xm_(1:ms,1:ks) = x0(2);
    xa_(1:ms,1:ks) = 1-x0(1)-x0(2);

    % combine distribution and weights
    vwR = v1.*wR;
    vwI = v2.*wI;
    xp(1) = (xp_*vwR)'*vwI;
    xm(1) = (xm_*vwR)'*vwI;

    % scale
    scale = (ones(ms,ks)*vwR)'*vwI;
    d1d2 = (ones(ms,ks)*vwR)'*vwI;
    d1d1d2 = (ones(ms,ks)*(vwR.*v1))'*vwI;
    d1d2d2 = (ones(ms,ks)*vwR)'*(vwI.*v2);
    % ode solver
    for i=2:length(t)

        % time step
        dt = t(i)-t(i-1);

        % calculate transition rates
        [ppa pap pma pam] = pxx_he(temp,stress(i) + xI,params.sigmaL,xR,...
            params.deltaSigmaTemp,params.tempL,params.Ea,params.Em,params.epsT,...
            params.tau,params.V);

        % implicit step 
        denom = 1+dt.*(pma+pam +ppa +pap) + dt.^2.*(ppa.*pam+ppa.*pam+pap.*pma);
        xm2_ = -dt.*pam.*xp_+dt.*pam + dt.^2.*ppa.*pam+xm_+dt.*ppa.*xm_+dt.*pap.*xm_;
        xm2_ = xm2_./denom;
        xp2_ = dt.*pma.*xp_-dt.*pap.*xm_+dt.^2.*pap.*pma+xp_+dt.*pap+dt.*pam.*xp_;
        xp2_ = xp2_./denom;

        xp_ = xp2_;
        xm_ = xm2_;
        xa_ = 1-xp_ - xm_;

        strain_ = sma_strain(stress(i),xp_,xm_,xa_,params.Ea,params.Em,params.epsT);
        xp(i) = (xp_*vwR)'*vwI/scale;
        xm(i) = (xm_*vwR)'*vwI/scale;
        strain(i) = (strain_*vwR)'*vwI/scale;

        % jacobian
        for ii =1:nAlpha
            J(i+offset,ii) =-(1/d1d2)*((strain_*(phi(:,ii).*wR))'*vwI)+...
            (1/d1d2)*strain(i)/sum(alpha);

        end
        for ii=1:nBeta
            J(i+offset,ii+nAlpha) = -(1/d1d2)*((strain_*vwR)'*(psi(:,ii).*wI))+...
            (1/d1d2)*strain(i)/sum(beta);

        end
        
        f(i+offset,1) = exp_strain{mm}(i) - strain(i);

    end

    % calculate xa at the end
    xa = 1-xp-xm;
    
    % offset for jacobian and residual (if using multiple temperatures)
    offset = length(stress);
end
