%--------------------------------------------------------------------------------------------------%
%
% function [f J] = costfunctionRes(x,exp_stress,params,xScale,exp_res,phase_fractions)
%   calculates the residual (resistance error) and jacobian for the optimization of both
%   densities and model parameters
%
%   Parameters:
%       x - design variables (rhoA0,rhoM0,alphaA,alphaM,nu)
%       exp_stress - experimental stress vectors (cell array)
%       params - hem sma parameters (struct)
%       xScale - scale values for design variables
%       exp_res - experimental resistance vectors (cell array)
%       phase_fractions - simulated phasefractions (cell array) 
%
%   Returns: 
%       f - vector of residuals
%       J - Jacobian
%
%   Required files: pxx_he.m, sma_strain.m, resistance.m, resistance_partials.m
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%

function [f J] = costfunctionRes(x,exp_stress,exp_temp,params,xScale,exp_res,phase_fractions)

offset = 0;
% initialize jacobian and residuals
ntotal = length(exp_stress{1})+length(exp_stress{2});
f = zeros(ntotal,1);
J = zeros(ntotal,length(x));


for mm = 1:length(exp_stress)
    
    % extract parameters
    nu = x(5)*xScale(5);
    r_sma = params.r_sma;
    rhoM0 = x(2)*xScale(2);
    rhoA0 = x(1)*xScale(1);
    alphaA = x(3)*xScale(3);
    alphaM = x(4)*xScale(4);
    L0 = params.L0;
    tempM = params.tempM;
    tempA = params.tempA;

    % extract phase fractions
    xp = phase_fractions{mm}.xp;
    xm = phase_fractions{mm}.xm;
    xa = phase_fractions{mm}.xa;
    
    % extract stress and temp
    stress = exp_stress{mm};
    temp = exp_temp{mm};
    
    strain = sma_strain(stress,xp,xm,xa,params.Ea,params.Em,params.epsT);
    
    res = resistance(strain,stress,temp,xp,xm,xa,params.Em,params.Ea,rhoM0,rhoA0,alphaA,alphaM,tempA,tempM,L0,r_sma,nu);
    
    [p1 p2 p3 p4 p5] = resistance_partials(strain,stress,temp,xp,xm,xa,...
            params.Em,params.Ea,rhoM0,rhoA0,alphaA,alphaM,tempA,tempM,L0,r_sma,nu);
    
    % x(1) = rhoA0
    % x(2) = rhoM0 
    % x(3) = alphaA
    % x(4) = alphaM
    % x(5) = nu  
    J(1+offset:offset+length(exp_stress{mm}),1) = p1*xScale(1);
    J(1+offset:offset+length(exp_stress{mm}),2) = p2*xScale(2);
    J(1+offset:offset+length(exp_stress{mm}),3) = p3*xScale(3);
    J(1+offset:offset+length(exp_stress{mm}),4) = p4*xScale(4);
    J(1+offset:offset+length(exp_stress{mm}),5) = p5*xScale(5);    
    % residual
    f(1+offset:offset+length(exp_stress{mm}),1) = exp_res{mm} - res;

    % calculate offset for residuals and jacobian
    offset = length(stress);
end