%--------------------------------------------------------------------------------------------------%
%
% function [f,J] = costfunctionSS(x,exp_t,ic,exp_stress, exp_temp,params,xScale,exp_strain)
%   calculates the residual (strain error) and jacobian for the optimization of both
%   densities and model parameters
%
%   Parameters:
%       x - design variables (alpha, beta, Ea, Em, sigmaL, deltaSigmaTemp, epsT, tau, V)
%       exp_t - experimental time vectors (cell array)
%       ic - initial conditions (cell array)
%       exp_stress - experimental stress vectors (cell array)
%       params - hem sma parameters (struct)
%       xScale - scale values for design variables
%       exp_stress - experimental strain vectors (cell array)
%       
%
%   Returns: 
%       f - vector of residuals
%       J - Jacobian
%
%   Required files: subinterval_pts.m, AlphaLogNormal.m, AlphaNormal.m,
%   pxx_he.m, sma_strain.m, pxx_he_partials.m, pxx_he_Deeppartials.m
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%

function [f,J] = costfunctionSS(x,exp_t,ic,exp_stress,...
    exp_temp,params,xScale,exp_strain)

nAlpha = length(params.sigmaRmean);
nBeta = length(params.b1);

% extract paramters
alpha = x(1:nAlpha);
beta = x(nAlpha+1:nAlpha+nBeta);

% extract density parameters
sigmaRmean = params.sigmaRmean;
c1 = params.c1;
b1 = params.b1;

% range of integration on sigmaR
medians_ = exp(sigmaRmean);
sds_ = sqrt((exp(c1.^2)-1).*exp(2*sigmaRmean+c1.^2));
lbR_ = max(medians_-2.*sds_,zeros(length(sigmaRmean),1));
ubR_ = medians_+2.*sds_;
lbR = min(lbR_);
ubR = max(ubR_);


% range on sigmaI
sigmaImax = 2*max(b1);

% set up intervals
[xR wR] = subinterval_pts(params.Npts,params.NsubInts,lbR,ubR);
[xI wI] = subinterval_pts(params.Npts,params.NsubInts,-sigmaImax,sigmaImax);

% scale points
wR = wR.*(ubR-lbR)/params.NsubInts;
wI = wI*sigmaImax*2/params.NsubInts;
% calculate distributions
v1 = AlphaLogNormal(xR,alpha,sigmaRmean,c1)/(sum(alpha));
v2 = AlphaNormal(xI,beta,b1)/(sum(beta));
% combine distribution and weights
vwR = v1.*wR;
vwI = v2.*wI;
% scale
ms = length(xI);
ks = length(xR);
scale = (ones(ms,ks)*vwR)'*vwI;

% kernels
for i =1:nAlpha
    phi(:,i) = AlphaLogNormal(xR,1,sigmaRmean(i),c1(i));
end
for j=1:nBeta
    psi(:,j) = AlphaNormal(xI,1,b1(j));
end
offset = 0;

% initialize jacobian and residuals
ntotal = length(exp_stress{1})+length(exp_stress{2});
f = zeros(ntotal,1);
J = zeros(ntotal,length(x));

for mm = 1:length(exp_stress)
    
    stress = exp_stress{mm};
    temp = exp_temp{mm};
    x0 = ic{mm};
    t = exp_t{mm};
    % check to make sure stress is a column vector
    [nstress mstress] = size(stress);
    if(nstress == 1)
        stress = stress'; % make a column vector
    end

    % make a matrix
    stress_ = zeros(length(stress),length(xI));

    for i=1:length(xI)
        stress_(:,i) = stress+xI(i);
    end
    % optimization SS
    % x(len+1) = Ea;
    % x(len+2) = Em
    % x(len+3) = sigmaL
    % x(len+4) = deltaSigmaTemp
    % x(len+5) = epsT
    % x(len+6) = taux
    % x(len+7) = V
    
    indX = nAlpha+nBeta;

    % sma constitutive model parameters
    extemp = params.extemp;                 % external temp [K]
    epsT = x(indX+5)*xScale(5);                     % maximum rec strain
    Ea = x(indX+1)*xScale(1);                         % austenite young's mod [Pa]
    Em = x(indX+2)*xScale(2);                         % martensite young's mod [Pa]
    tempL = params.tempL;                   % low stress tranisition temp [K]            
    sigmaL = x(indX+3)*xScale(3);                 % transition stress [Pa]
    deltaSigmaTemp = x(indX+4)*xScale(4); % hysteresis loop temp dependence [Pa/K]
    vd = x(indX+7)*xScale(7);                       	% volume layer [m^2]             
    tau =x(indX+6)*xScale(6);                     % relaxation time [1/s]

    % size of vectors
    ns = length(t);
    

    % initialize output  (xp, xm, xa)
    xp = zeros(ns,1);
    xa = zeros(ns,1);
    xm = zeros(ns,1);
    strain = zeros(ns,1);
    xp_(1:ms,1:ks) = x0(1);
    xm_(1:ms,1:ks) = x0(2);
    xa_(1:ms,1:ks) = 1-x0(1)-x0(2);

    % ic
    xp(1) = (xp_*vwR)'*vwI;
    xm(1) = (xm_*vwR)'*vwI;

    % coefficients for derivatives
    d1d2 = (ones(ms,ks)*vwR)'*vwI;
    d1d1d2 = (ones(ms,ks)*(vwR.*v1))'*vwI;
    d1d2d2 = (ones(ms,ks)*vwR)'*(vwI.*v2);
    % variables for derivatives
    xptau = zeros(ms,ks);
    xmtau = zeros(ms,ks);
    xpV = zeros(ms,ks);
    xmV = zeros(ms,ks);
    xpSigmaL = zeros(ms,ks);
    xmSigmaL = zeros(ms,ks);
    xpDST = zeros(ms,ks);
    xmDST = zeros(ms,ks);
    
    % ode solver
    for i=2:length(t)

        % time step
        dt = t(i)-t(i-1);

        % calculate transition rates
        [ppa pap pma pam] = pxx_he(temp,stress(i) + xI,sigmaL,xR,deltaSigmaTemp,tempL,Ea,Em,epsT,tau,vd);

        % implicit step 
        denom = 1+dt.*(pma+pam +ppa +pap) + dt.^2.*(ppa.*pam+ppa.*pam+pap.*pma);
        xm2_ = -dt.*pam.*xp_+dt.*pam + dt.^2.*ppa.*pam+xm_+dt.*ppa.*xm_+dt.*pap.*xm_;
        xm2_ = xm2_./denom;
        xp2_ = dt.*pma.*xp_-dt.*pap.*xm_+dt.^2.*pap.*pma+xp_+dt.*pap+dt.*pam.*xp_;
        xp2_ = xp2_./denom;

        xp_ = xp2_;
        xm_ = xm2_;
        xa_ = 1-xp_ - xm_;

        strain_ = sma_strain(stress(i),xp_,xm_,xa_,Ea,Em,epsT);
        xp(i) = (xp_*vwR)'*vwI/scale;
        xm(i) = (xm_*vwR)'*vwI/scale;
        strain(i) = (strain_*vwR)'*vwI/scale;

        % jacobian for density coefficients
        for ii =1:nAlpha
            J(i+offset,ii) =-(1/d1d2)*((strain_*(phi(:,ii).*wR))'*vwI)+...
            (1/d1d2)*strain(i)/sum(alpha);

        end
        for ii=1:nBeta
            J(i+offset,ii+nAlpha) = -(1/d1d2)*((strain_*vwR)'*(psi(:,ii).*wI))+...
            (1/d1d2)*strain(i)/sum(beta);

        end
        
        % x(len+1) = Ea;
        % x(len+2) = Em
        % x(len+3) = sigmaL
        % x(len+4) = deltaSigmaTemp
        % x(len+5) = epsT
        % x(len+6) = taux
        % x(len+7) = V

         % Ea jacobian
        J(i+offset,indX+1) = xa(i)*stress(i)/Ea^2*xScale(1);

        % Em jacobian
        J(i+offset,indX+2) = stress(i)*(xp(i)+xm(i))/Em^2*xScale(2);
        
        % epsT jacobian
        J(i+offset,indX+5) = (-xp(i)+xm(i))*xScale(5);
        
        
        % partials jacobian
        [pxx_V pxx_taux] = pxx_he_partials(temp,stress(i) + xI,sigmaL,xR,deltaSigmaTemp,tempL,Ea,Em,epsT,tau,vd);
        [pxx_sigmaL pxx_deltaSigmaTemp] = pxx_he_Deeppartials(temp,stress(i) + xI,sigmaL,xR,deltaSigmaTemp,tempL,Ea,Em,epsT,tau,vd);

        
        % sigmaL jacobian
        dxp_ = xpSigmaL;
        dxm_ = xmSigmaL;
        coeff=1./(dt^2.*pam.*pap+dt.*ppa+ones(ms,ks)+dt.*pap+2.*dt^2.*pam.*ppa+2.*dt.*pam);
        dpap = pxx_sigmaL{1};
        dppa = pxx_sigmaL{2};
        dpam = pxx_sigmaL{3};
        dpma = pxx_sigmaL{4};
        dxm2_ = dt.*dpma.*xm_+dt*dpam.*xp_+dt.*dpam*xm_+dt^2*ppa.*dpma.*xm_...
            +dt^2.*ppa.*dpam.*xp_+dt^2.*ppa.*dpam.*xm_-dt^2.*ppa.*dpam-dt.*ppa.*dxm_...
            +dt^2.*dpma.*xm_.*pap+dt^2.*dpam.*xp_.*pap+dt^2.*dpam.*xm_.*pap...
            +dt^2.*pam.*dpap.*xa_+dt.*pam.*dxp_-dt.*dpam-dxm_...
            -dt^2.*dpam.*pap-dxm_.*dt.*pap;
        dxp2_ = -dt^2.*dpam.*pap-dxm_.*dt.*pap+dt^2.*dpma*xm_.*pap+dt^2.*dpam.*xp_.*pap...
            +dt^2.*dpam.*xm_.*pap+dt.*dpap.*xa_+dxp_-dt.*dppa.*xp_+2.*dt^2.*pam.*dpap.*xa_...
            +2.*dt.*pam.*dxp_-2.*dt^2.*pam.*dppa.*xp_;

        dxm2_ =-1*coeff.*dxm2_;
        dxp2_ = coeff.*dxp2_;

        xpSigmaL = dxp2_;
        xmSigmaL= dxm2_;

        dxp = (xpSigmaL*vwR)'*vwI;
        dxm = (xmSigmaL*vwR)'*vwI;
        J(i+offset,indX+3) = -1*(dxp*(stress(i)/Em-stress(i)/Ea+epsT) + dxm*(stress(i)/Em-stress(i)/Ea-epsT))*xScale(3);
        
        % deltaSigmaTemp jacobian
        dxp_ = xpDST;
        dxm_ = xmDST;
        coeff=1./(dt^2.*pam.*pap+dt.*ppa+ones(ms,ks)+dt.*pap+2.*dt^2.*pam.*ppa+2.*dt.*pam);
        dpap = pxx_deltaSigmaTemp{1};
        dppa = pxx_deltaSigmaTemp{2};
        dpam = pxx_deltaSigmaTemp{3};
        dpma = pxx_deltaSigmaTemp{4};
        dxm2_ = dt.*dpma.*xm_+dt*dpam.*xp_+dt.*dpam*xm_+dt^2*ppa.*dpma.*xm_...
            +dt^2.*ppa.*dpam.*xp_+dt^2.*ppa.*dpam.*xm_-dt^2.*ppa.*dpam-dt.*ppa.*dxm_...
            +dt^2.*dpma.*xm_.*pap+dt^2.*dpam.*xp_.*pap+dt^2.*dpam.*xm_.*pap...
            +dt^2.*pam.*dpap.*xa_+dt.*pam.*dxp_-dt.*dpam-dxm_...
            -dt^2.*dpam.*pap-dxm_.*dt.*pap;
        dxp2_ = -dt^2.*dpam.*pap-dxm_.*dt.*pap+dt^2.*dpma*xm_.*pap+dt^2.*dpam.*xp_.*pap...
            +dt^2.*dpam.*xm_.*pap+dt.*dpap.*xa_+dxp_-dt.*dppa.*xp_+2.*dt^2.*pam.*dpap.*xa_...
            +2.*dt.*pam.*dxp_-2.*dt^2.*pam.*dppa.*xp_;

        dxm2_ =-1*coeff.*dxm2_;
        dxp2_ = coeff.*dxp2_;

        xpDST = dxp2_;
        xmDST = dxm2_;

        dxp = (xpDST*vwR)'*vwI;
        dxm = (xmDST*vwR)'*vwI;
        J(i+offset,indX+4) = -1*(dxp*(stress(i)/Em-stress(i)/Ea+epsT) + dxm*(stress(i)/Em-stress(i)/Ea-epsT))*xScale(4);
        
        % taux jacobian
        dxp_ = xptau;
        dxm_ = xmtau;
        coeff=1./(dt^2.*pam.*pap+dt.*ppa+ones(ms,ks)+dt.*pap+2.*dt^2.*pam.*ppa+2.*dt.*pam);
        dpap = pxx_taux{1};
        dppa = pxx_taux{2};
        dpam = pxx_taux{3};
        dpma = pxx_taux{4};
        dxm2_ = dt.*dpma.*xm_+dt*dpam.*xp_+dt.*dpam*xm_+dt^2*ppa.*dpma.*xm_...
            +dt^2.*ppa.*dpam.*xp_+dt^2.*ppa.*dpam.*xm_-dt^2.*ppa.*dpam-dt.*ppa.*dxm_...
            +dt^2.*dpma.*xm_.*pap+dt^2.*dpam.*xp_.*pap+dt^2.*dpam.*xm_.*pap...
            +dt^2.*pam.*dpap.*xa_+dt.*pam.*dxp_-dt.*dpam-dxm_...
            -dt^2.*dpam.*pap-dxm_.*dt.*pap;
        dxp2_ = -dt^2.*dpam.*pap-dxm_.*dt.*pap+dt^2.*dpma*xm_.*pap+dt^2.*dpam.*xp_.*pap...
            +dt^2.*dpam.*xm_.*pap+dt.*dpap.*xa_+dxp_-dt.*dppa.*xp_+2.*dt^2.*pam.*dpap.*xa_...
            +2.*dt.*pam.*dxp_-2.*dt^2.*pam.*dppa.*xp_;

        dxm2_ =-1*coeff.*dxm2_;
        dxp2_ = coeff.*dxp2_;

        xptau = dxp2_;
        xmtau = dxm2_;

        dxp = (xptau*vwR)'*vwI;
        dxm = (xmtau*vwR)'*vwI;
        J(i+offset,indX+6) = -1*(dxp*(stress(i)/Em-stress(i)/Ea+epsT) + dxm*(stress(i)/Em-stress(i)/Ea-epsT))*xScale(6);
        
        % V jacobian
        dxp_ = xpV;
        dxm_ = xmV;
        coeff=1./(dt^2.*pam.*pap+dt.*ppa+ones(ms,ks)+dt.*pap+2.*dt^2.*pam.*ppa+2.*dt.*pam);
        dpap = pxx_V{1};
        dppa = pxx_V{2};
        dpam = pxx_V{3};
        dpma = pxx_V{4};
        dxm2_ = dt.*dpma.*xm_+dt*dpam.*xp_+dt.*dpam*xm_+dt^2*ppa.*dpma.*xm_...
            +dt^2.*ppa.*dpam.*xp_+dt^2.*ppa.*dpam.*xm_-dt^2.*ppa.*dpam-dt.*ppa.*dxm_...
            +dt^2.*dpma.*xm_.*pap+dt^2.*dpam.*xp_.*pap+dt^2.*dpam.*xm_.*pap...
            +dt^2.*pam.*dpap.*xa_+dt.*pam.*dxp_-dt.*dpam-dxm_...
            -dt^2.*dpam.*pap-dxm_.*dt.*pap;
        dxp2_ = -dt^2.*dpam.*pap-dxm_.*dt.*pap+dt^2.*dpma*xm_.*pap+dt^2.*dpam.*xp_.*pap...
            +dt^2.*dpam.*xm_.*pap+dt.*dpap.*xa_+dxp_-dt.*dppa.*xp_+2.*dt^2.*pam.*dpap.*xa_...
            +2.*dt.*pam.*dxp_-2.*dt^2.*pam.*dppa.*xp_;

        dxm2_ =-1*coeff.*dxm2_;
        dxp2_ = coeff.*dxp2_;

        xpV = dxp2_;
        xmV = dxm2_;

        dxp = (xpV*vwR)'*vwI;
        dxm = (xmV*vwR)'*vwI;
        J(i+offset,indX+7) = -1*(dxp*(stress(i)/Em-stress(i)/Ea+epsT) + dxm*(stress(i)/Em-stress(i)/Ea-epsT))*xScale(7);
        
        % RESIDUALS
        f(i+offset,1) = exp_strain{mm}(i) - strain(i);

    end

    % calculate offset for residuals and jacobian
    offset = length(stress);
end
