%--------------------------------------------------------------------------------------------------%
%
% function y = extract_densities(params)
%   appends the density parameters to the struct params
%
%   Parameters:
%       params - struct containing current hem sma parameters
%
%   Returns: 
%       y - struct containing all information in params plus density
%           parameters
%
%   Required files: subinterval_pts.m, AlphaLogNormal.m, AlphaNormal.m
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%

function y = extract_densities(params)

% range of integration on sigmaR
medians_ = exp(params.sigmaRmean);
sds_ = sqrt((exp(params.c1.^2)-1).*exp(2*params.sigmaRmean+params.c1.^2));
lbR_ = max(medians_-2.*sds_,zeros(length(params.sigmaRmean),1));
ubR_ = medians_+2.*sds_;
lbR = min(lbR_);
ubR = max(ubR_);


% range on sigmaI
sigmaImax = 2*max(params.b1);

% set up intervals
[xR wR] = subinterval_pts(params.Npts,params.NsubInts,lbR,ubR);
[xI wI] = subinterval_pts(params.Npts,params.NsubInts,-sigmaImax,sigmaImax);

% scale points
wR = wR.*(ubR-lbR)/params.NsubInts;
wI = wI*sigmaImax*2/params.NsubInts;
% calculate distributions
v1 = AlphaLogNormal(xR,params.alpha,params.sigmaRmean,params.c1)/(sum(params.alpha));
v2 = AlphaNormal(xI,params.beta,params.b1)/(sum(params.beta));
% combine distribution and weights
vwR = v1.*wR;
vwI = v2.*wI;
% scale
ms = length(xI);
ks = length(xR);
scale = (ones(ms,ks)*vwR)'*vwI;

% appped to params
params.scale = scale;
params.xI = xI;
params.xR = xR;
params.vwI = vwI;
params.vwR = vwR;

y = params;