%--------------------------------------------------------------------------------------------------%
%
% function params = load_initial_parameters()
%   initializes sma model parameters
%
%   Parameters:
%       none
%
%   Returns: 
%       params - a struct containing the hem sma parameters
%
%   Required files: extract_densitities.m
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%
function params = load_initial_parameters()

% initialize SMA parameters 
params.extemp = 298;            % External temperature [K]
params.Ea = 35e9;               % Austenitic young's modulus [Pa]
params.Em = 28e9;               % Martensitic young's modulus [Pa]
params.tempL = 348;             % lower transition temperature [K]
params.sigmaL = 275e6;          % lower transition stress [Pa]
params.deltaSigmaTemp = 10e6;   % hysteresis loop temperature dependence [Pa/K]
params.tau = 1.09;             % relaxation time [s]
params.V = 8e-26;               % layer volume [m^3] 
params.epsT = 0.046;            % maxium recoverable strain []
params.L0 = 169/1000;           % SMA wire length [m]
params.rhoM0 = 10.71e-7;        % resistivity of martensite
params.rhoA0 = 8.82e-7;         % resistivity of austenite
params.alphaA = 0.0003;         % temp dependence of austenite resistivity
params.alphaM = 0.0003;         % temp dependence of martensite resistivity
params.nu = 0.67;               % poison's ratio
params.r_sma =6.35e-5;          % sma wire radius [m]
params.tempM = 298;             % nominal temp for martensite resistivity [K]
params.tempA = 273+70;          % nominal temp for austenite resistivity [K]

% integration parameters
params.NsubInts = 20;
params.Npts = 4;

% experimentally estimated relative stress density parameters
muR = 148.5e6;
sigmaRmean_ = muR + [-75e6; -75e6; 0e6; 0e6; 75e6; 75e6];
c1_ = (30.5e6)^2*[1; 5; 1; 5; 1; 5];
sigmaRmean = log(sigmaRmean_)-0.5*log(1+c1_./(sigmaRmean_).^2);
c1 = log(1+c1_./(sigmaRmean_).^2);

% experimentally determined interaction stress density parameters
b1 = [0.25; 0.5; 1; 2; 4].*35.4e6;

% append to params
params.b1 = b1;
params.c1 = c1;
params.sigmaRmean = sigmaRmean;

% initial coefficients
alpha = [0.1; 0.1; 0.5; 0.1; 0.1; 0.1;];
beta = [0.4; 0.4; 0.4; 0.2; 0.1];
alpha = alpha/sum(alpha);
beta = beta/sum(beta);

% append to params
params.alpha = alpha;
params.beta = beta;

% initial densities
params = extract_densities(params);