%--------------------------------------------------------------------------------------------------%
%
% function [pxx_sigmaL pxx_deltaSigmaTemp]= pxx_he_Deeppartials(temp,sigma,sigmaL,deltaSigma,deltaSigmaTemp,tempL,Ea,Em,epsT,taux,V)
%   calculates partials of transition liklihoods
%
%   Parameters:
%       temp - temperature (scalar)
%       sigma - stress (vector)
%       deltaSigma - relative stress (vector)
%       sigmaL..V - sma model parameters (see load_initial_parameters.m)
%
%   Returns: 
%       pxx_sigmaL - cell array containing derivatives of transition rates wrt sigmaL
%       pxx_deltaSigmaTemp- cell array containing derivatives of transition rates wrt deltaSigmaTemp
%
%   Required files: none
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%


function [pxx_sigmaL pxx_deltaSigmaTemp]= pxx_he_Deeppartials(temp,sigma,sigmaL,deltaSigma,deltaSigmaTemp,tempL,Ea,Em,epsT,taux,V)

% boltzman constant
kb = 1.38044e-23;  

% delta sigma needs to vary across columns, sigma needs to vary across rows
ns = length(sigma);
ms = length(deltaSigma);
% expressions to determine the transition probabilities
sigmaA(1:ns,1:ms) = sigmaL+deltaSigmaTemp*(temp-tempL);
deltaSigma_ = repmat(deltaSigma',[ns 1]); % make a matrix

sigmaM = sigmaA - deltaSigma_;
wa  = sigmaA/Ea;
wm  = sigmaM/Em+epsT;
deltawam = wa-wm;
deltaBeta = (1/2)*(Em*(epsT-wa).*(epsT-wm)-Ea.*wm.*wa);
a1(1:ns,1:ms) = Em/2;
a2 = (Em*(epsT-wm)+Ea*wa)./(2*(deltawam));
a3 = Ea/2;
a4 = a2;
a5 = a1;
b1(1:ns,1:ms) = Em*epsT;
b2 = wa.*(Em*(epsT-wm)+Ea*wm)./(deltawam);
b3(1:ns,1:ms) = 0;
b4 = -b2;
b5 = -b1;
c1(1:ns,1:ms) = (Em/2)*(epsT^2);
c2 = (2.*deltaBeta.*(deltawam)+(wa.^2).*(Em.*(epsT-wm)+Ea.*wm))./(2.*(deltawam));
c3 = deltaBeta;
c4 = c2;
c5 = c1;

% scale sigma
m_sigma = repmat(sigma,[1 ms]);

eps1 = (m_sigma - b1)./(2*a1);
eps2 = (m_sigma - b2)./(2*a2);
eps3 = (m_sigma - b3)./(2*a3);
eps4 = (m_sigma - b4)./(2*a4);
eps5 = (m_sigma - b5)./(2*a5);

% intermediate steps
da2_sL = -deltaSigma_.*(1/Ea-1/Em)./(2*deltawam.^2);    
da4_sL = da2_sL;                                      


db2_sL = (-sigmaM+Ea.*wm)./(Ea.*(deltawam)) + (sigmaA.*(-1+Ea/Em))./(Ea.*deltawam) ...
    -(sigmaA.*(-sigmaM+Ea.*wm)*(1/Ea-1/Em))./(Ea*deltawam.^2); 
db4_sL = -1*db2_sL;

dbeta_sL = 0.5*(sigmaA+sigmaM)*(1/Ea-1/Em)-epsT;       
dL_sL = (sigmaA.*(-sigmaM+Ea.*wm))./(Ea^2.*deltawam)...
    + (0.5*sigmaA.^2.*(-1+Ea/Em))./(Ea^2.*deltawam)...
    - (0.5*sigmaA.^2.*(-sigmaM+Ea*wm).*(1/Ea-1/Em))./(Ea.^2.*deltawam.^2); 
dc2_sL = dbeta_sL +dL_sL;  

dc3_sL = dbeta_sL;
dc4_sL = dc2_sL;

deps2_sL = -0.5*(db2_sL./a2+(m_sigma-b2).*da2_sL./(a2.^2));
deps4_sL = -0.5*(db4_sL./a4+(m_sigma-b4).*da4_sL./(a4.^2));


deps2_DST = -0.5*(db2_sL.*(temp-tempL)./a2+(m_sigma-b2).*da2_sL.*(temp-tempL)./(a2.^2));
deps4_DST = -0.5*(db4_sL.*(temp-tempL)./a4+(m_sigma-b4).*da4_sL.*(temp-tempL)./(a4.^2));


dG1_sL = 0;
dG3_sL = dc3_sL;
dG5_sL = 0;

dG1_DST = 0;
dG3_DST = dc3_sL.*(temp-tempL);
dG5_DST= 0;

% limit parabola 2
test2 = ( (eps2 > -wa) | (eps2 < -wm) );
eps2 = min(max((m_sigma - b2)./(2*a2),-wm),-wa);
dG2_sL = (da2_sL.*eps2.^2+2.*a2.*eps2.*deps2_sL+db2_sL.*eps2+b2.*deps2_sL+dc2_sL-m_sigma.*deps2_sL).*(~test2) +...
    (da2_sL.*eps2.^2+db2_sL.*eps2+dc2_sL).*test2;
dG2_DST = (da2_sL.*(temp-tempL).*eps2.^2+2.*a2.*eps2.*deps2_DST+db2_sL.*(temp-tempL).*eps2+b2.*deps2_DST+dc2_sL.*(temp-tempL)-m_sigma.*deps2_DST).*(~test2) +...
    (da2_sL.*(temp-tempL).*eps2.^2+db2_sL.*(temp-tempL).*eps2+dc2_sL.*(temp-tempL)).*test2;

% limit parabola 4
test4 = (eps4 > wm) | (eps4 < wa);
eps4 = min(max((m_sigma - b4)./(2*a4),wa),wm);
dG4_sL = (da4_sL.*eps4.^2+2.*a4.*eps4.*deps4_sL+db4_sL.*eps4+b4.*deps4_sL+dc4_sL-m_sigma.*deps4_sL).*(~test4)...
    +(da4_sL.*eps4.^2+db4_sL.*eps4+dc4_sL).*test4;
dG4_DST = (da4_sL.*(temp-tempL).*eps4.^2+2.*a4.*eps4.*deps4_DST+db4_sL.*(temp-tempL).*eps4+b4.*deps4_DST+dc4_sL.*(temp-tempL)-m_sigma.*deps4_DST).*(~test4)...
    +(da4_sL.*(temp-tempL).*eps4.^2+db4_sL.*(temp-tempL).*eps4+dc4_sL.*(temp-tempL)).*test4;

eps1 = min((m_sigma - b1)./(2*a1),-wm);
eps3 = min(max((m_sigma - b3)./(2*a3),-wa),wa);
eps5 = max((m_sigma - b5)./(2*a5),wm);

G1 = a1.*eps1.^2+b1.*eps1+c1-m_sigma.*eps1;
G2 = a2.*eps2.^2+b2.*eps2+c2-m_sigma.*eps2;
G3 = a3.*eps3.^2+b3.*eps3+c3-m_sigma.*eps3;
G4 = a4.*eps4.^2+b4.*eps4+c4-m_sigma.*eps4;
G5 = a5.*eps5.^2+b5.*eps5+c5-m_sigma.*eps5;

dGmA = dG2_sL;
dGAm = dG2_sL - dG3_sL;
dGAp = dG4_sL - dG3_sL;
dGpA = dG4_sL;

dGmA2 = dG2_DST;
dGAm2 = dG2_DST - dG3_DST;
dGAp2 = dG4_DST - dG3_DST;
dGpA2 = dG4_DST;



zz1 = 0;
deltaGmA = max((G2-G1),zz1);
deltaGAm = max((G2-G3),zz1);
deltaGAp = max((G4-G3),zz1);
deltaGpA = max((G4-G5),zz1);


lambdaT = V/(kb*temp);
tauxinv = 1/taux;

% transition probabilities
pap = tauxinv*exp(-lambdaT*deltaGAp);
ppa = tauxinv*exp(-lambdaT*deltaGpA);
pam = tauxinv*exp(-lambdaT*deltaGAm);
pma = tauxinv*exp(-lambdaT*deltaGmA);

pap_sigmaL = -lambdaT*pap.*dGAp;
ppa_sigmaL  = -lambdaT*ppa.*dGpA;
pam_sigmaL  = -lambdaT*pam.*dGAm;
pma_sigmaL  = -lambdaT*pma.*dGmA;

pap_deltaSigmaTemp = -1*lambdaT*pap.*dGAp2;
ppa_deltaSigmaTemp = -1*lambdaT*ppa.*dGpA2;
pam_deltaSigmaTemp  = -1*lambdaT*pam.*dGAm2;
pma_deltaSigmaTemp  = -1*lambdaT*pma.*dGmA2;

pxx_sigmaL{1} = pap_sigmaL;
pxx_sigmaL{2} = ppa_sigmaL;
pxx_sigmaL{3} = pam_sigmaL;
pxx_sigmaL{4} = pma_sigmaL;

pxx_deltaSigmaTemp{1} = pap_deltaSigmaTemp ;
pxx_deltaSigmaTemp{2} = ppa_deltaSigmaTemp ;
pxx_deltaSigmaTemp{3} = pam_deltaSigmaTemp ;
pxx_deltaSigmaTemp{4} = pma_deltaSigmaTemp ;

