%--------------------------------------------------------------------------------------------------%
%
% function [pxx_V pxx_tau]= pxx_he_partials(temp,sigma,sigmaL,deltaSigma,deltaSigmaTemp,tempL,Ea,Em,epsT,taux,V,dt)
%   calculates partials of transition liklihoods
%
%   Parameters:
%       temp - temperature (scalar)
%       sigma - stress (vector)
%       deltaSigma - relative stress (vector)
%       sigmaL..V - sma model parameters (see load_initial_parameters.m)
%
%   Returns: 
%       pxx_V - cell array containing derivatives of transition rates wrt V
%       pxx_tau - cell array containing derivatives of transition rates wrt tau
%
%   Required files: none
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%

function [pxx_V pxx_tau]= pxx_he_partials(temp,sigma,sigmaL,deltaSigma,deltaSigmaTemp,tempL,Ea,Em,epsT,taux,V)

% boltzman constant
kb = 1.38044e-23;  

% delta sigma needs to vary across columns, sigma needs to vary across rows
ns = length(sigma);
ms = length(deltaSigma);
% expressions to determine the transition probabilities
sigmaA(1:ns,1:ms) = sigmaL+deltaSigmaTemp*(temp-tempL);
deltaSigma_ = repmat(deltaSigma',[ns 1]); % make a matrix
sigmaM = sigmaA - deltaSigma_;
wa  = sigmaA/Ea;
wm  = sigmaM/Em+epsT;
deltawam = wa-wm;
deltaBeta = (1/2)*(Em*(epsT-wa).*(epsT-wm)-Ea.*wm.*wa);
a1(1:ns,1:ms) = Em/2;
a2 = (Em*(epsT-wm)+Ea*wa)./(2*(deltawam));
a3 = Ea/2;
a4 = a2;
a5 = a1;
b1(1:ns,1:ms) = Em*epsT;
b2 = wa.*(Em*(epsT-wm)+Ea*wm)./(deltawam);
b3(1:ns,1:ms) = 0;
b4 = -b2;
b5 = -b1;
c1(1:ns,1:ms) = (Em/2)*(epsT^2);
c2 = (2.*deltaBeta.*(deltawam)+(wa.^2).*(Em.*(epsT-wm)+Ea.*wm))./(2.*(deltawam));
c3 = deltaBeta;
c4 = c2;
c5 = c1;

% scale sigma
m_sigma = repmat(sigma,[1 ms]);

eps1 = min((m_sigma - b1)./(2*a1),-wm);
eps2 = min(max((m_sigma - b2)./(2*a2),-wm),-wa);
eps3 = min(max((m_sigma - b3)./(2*a3),-wa),wa);
eps4 = min(max((m_sigma - b4)./(2*a4),wa),wm);
eps5 = max((m_sigma - b5)./(2*a5),wm);


G1 = a1.*eps1.^2+b1.*eps1+c1-m_sigma.*eps1;
G2 = a2.*eps2.^2+b2.*eps2+c2-m_sigma.*eps2;
G3 = a3.*eps3.^2+b3.*eps3+c3-m_sigma.*eps3;
G4 = a4.*eps4.^2+b4.*eps4+c4-m_sigma.*eps4;
G5 = a5.*eps5.^2+b5.*eps5+c5-m_sigma.*eps5;

zz1 = zeros(ns,ms);
deltaGmA = max((G2-G1),zz1);
deltaGAm = max((G2-G3),zz1);
deltaGAp = max((G4-G3),zz1);
deltaGpA = max((G4-G5),zz1);


lambdaT = V/(kb*temp);
tauxinv = 1/taux;

% transition probabilities
pap = tauxinv*exp(-lambdaT*deltaGAp);
ppa = tauxinv*exp(-lambdaT*deltaGpA);
pam = tauxinv*exp(-lambdaT*deltaGAm);
pma = tauxinv*exp(-lambdaT*deltaGmA);

% partials
pap_V = (-deltaGAp/(kb*temp)).*pap;
ppa_V  = (-deltaGpA/(kb*temp)).*ppa;
pam_V  = (-deltaGAm/(kb*temp)).*pam;
pma_V  = (-deltaGmA/(kb*temp)).*pma;

pap_taux = -tauxinv*pap;
ppa_taux  = -tauxinv*ppa;
pam_taux  = -tauxinv*pam;
pma_taux  = -tauxinv*pma;


pxx_V{1} = pap_V;
pxx_V{2} = ppa_V;
pxx_V{3} = pam_V;
pxx_V{4} = pma_V;

pxx_tau{1} = pap_taux;
pxx_tau{2} = ppa_taux;
pxx_tau{3} = pam_taux;
pxx_tau{4} = pma_taux;

