%--------------------------------------------------------------------------------------------------%
%
% function [xp xm xa strain res] = sma_hem_constanttemp(t,x0,stress,temp,params)
%   simulates the homgenized energy shape memory alloy model for constant
%   temperatures using the stress input
%
%   Parameters:
%       t - time (vector)
%       x0 - initial condition (xp0, xm0)
%       stress - stress ( vector)
%       temp - temperature (scalar)
%       params - struct containing model parameters
%  
%   Returns: 
%       xp - martensite plus phase fraction (vector)
%       xm - martensite minus phase fraction (vector)
%       xa - austenite phase fraction (vector)
%       strain - strain (vector)
%       res - resistance (vector)
%
%   Required files: pxx_he.m, sma_strain.m, resistance.m
%                               
%                       ----------------------------
%
% Copyright (C) <2011> by <Ralph Smith (rsmith@ncsu.edu), John Crews(jhcrews@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%


function [xp xm xa strain res] = sma_hem_constanttemp(t,x0,stress,temp,params)

% extract density parameters
xR = params.xR;
xI = params.xI;
scale = params.scale;
vwR = params.vwR;
vwI = params.vwI;

% check to make sure stress is a column vector
[ns ms] = size(stress);
if(ns == 1)
    stress = stress'; % make a column vector
end

% make a matrix
stress_ = zeros(length(stress),length(xI));

for i=1:length(xI)
    stress_(:,i) = stress+xI(i);
end


% size of 3d matrix
ns = length(t);
ms = length(xI);
ks = length(xR);

% initialize output  (xp, xm, xa)
xp = zeros(ns,1);
xa = zeros(ns,1);
xm = zeros(ns,1);
strain = zeros(ns,1);
xp_(1:ms,1:ks) = x0(1);
xm_(1:ms,1:ks) = x0(2);
xa_(1:ms,1:ks) = 1-x0(1)-x0(2);

% ic
xp(1) = (xp_*vwR)'*vwI;
xm(1) = (xm_*vwR)'*vwI;

% ode solver
for i=2:length(t)
    
    % time step
    dt = t(i)-t(i-1);
    
    % calculate transition rates
    [ppa pap pma pam] = pxx_he(temp,stress(i) + xI,params.sigmaL,xR,...
        params.deltaSigmaTemp,params.tempL,params.Ea,params.Em,params.epsT,...
        params.tau,params.V);
        
    % implicit step 
    denom = 1+dt.*(pma+pam +ppa +pap) + dt.^2.*(ppa.*pma+ppa.*pam+pap.*pma);
    xm2_ = -dt.*pam.*xp_+dt.*pam + dt.^2.*ppa.*pam+xm_+dt.*ppa.*xm_+dt.*pap.*xm_;
    xm2_ = xm2_./denom;
    xp2_ = dt.*pma.*xp_-dt.*pap.*xm_+dt.^2.*pap.*pma+xp_+dt.*pap+dt.*pam.*xp_;
    xp2_ = xp2_./denom;
    
    xp_ = xp2_;
    xm_ = xm2_;
    xa_ = 1-xp_ - xm_;
    
    strain_ = sma_strain(stress(i),xp_,xm_,xa_,params.Ea,params.Em,params.epsT);
    xp(i) = (xp_*vwR)'*vwI;
    xm(i) = (xm_*vwR)'*vwI;
    strain(i) = (strain_*vwR)'*vwI;
end

xp = xp/scale;
xm = xm/scale;
strain = strain/scale;

% calculate xa at the end
xa = 1-xp-xm;

res = resistance(strain,stress,temp,xp,xm,xa,params.Em,params.Ea,...
    params.rhoM0,params.rhoA0,params.alphaA,params.alphaM,params.tempA,...
    params.tempM,params.L0,params.r_sma,params.nu);

